@@ -736,7 +736,9 @@ class Codestream(object):
         data = struct.unpack_from('>HIIIIIIIIH', read_buffer)
 
         rsiz = data[0]
-        if rsiz not in _KNOWN_PROFILES:
+
+        # Bit 14 (16384) signifies HTJ2K (JPH)
+        if rsiz not in _KNOWN_PROFILES and not np.bitwise_and(rsiz, 16384):
             msg = f"Invalid profile: (Rsiz={rsiz})."
             warnings.warn(msg, UserWarning)
 
@@ -767,11 +769,10 @@ class Codestream(object):
             num_tiles_y = (xysiz[1] - xyosiz[1]) / (xytsiz[1] - xytosiz[1])
         except ZeroDivisionError:
             msg = (
-                f"Invalid tile specification:  "
-                f"size of {xytsiz[1]} x {xytsiz[0]}, "
-                f"offset of {xytosiz[1]} x {xytsiz[0]}."
+                f"Invalid tile specification in SIZ segment at byte offset "
+                f"{offset}:  tile size of {xytsiz[1]} x {xytsiz[0]}."
             )
-            warnings.warn(msg, UserWarning)
+            raise ZeroDivisionError(msg)
         else:
             numtiles = np.ceil(num_tiles_x) * np.ceil(num_tiles_y)
             if numtiles > 65535:
@@ -951,8 +952,8 @@ class COCsegment(Segment):
         Coding style for this component.
     spcoc : byte array
         Coding style parameters for this component.
-    precinct_size : list of tuples
-        Dimensions of precinct.
+    precinct_size : nx2 array
+        Dimensions of precincts.
 
     References
     ----------
@@ -971,7 +972,7 @@ class COCsegment(Segment):
         if len(self.spcoc) > 5:
             self.precinct_size = _parse_precinct_size(self.spcoc[5:])
         else:
-            self.precinct_size = ((32768, 32768))
+            self.precinct_size = np.array(((32768, 32768)))
 
         self.length = length
         self.offset = offset
@@ -993,7 +994,7 @@ class COCsegment(Segment):
             f'        Number of decomposition levels:  {self.spcoc[0]}\n'
             f'        Code block height, width:  ({width} x {height})\n'
             f'        Wavelet transform:  {xform}\n'
-            f'        Precinct size:  {self.precinct_size}\n'
+            f'        Precinct size:  {self.precinct_size.tolist()}\n'
             f'        {_context_string(self.spcoc[3])}'
         )
 
@@ -1028,8 +1029,8 @@ class CODsegment(Segment):
         Wavelet transform used
     cstyle : int
         Style of the code-block passes
-    precinct_size : list
-        2-tuples of precinct sizes.
+    precinct_size : nx2 array
+        Array of precinct sizes.
 
     References
     ----------
@@ -1071,9 +1072,9 @@ class CODsegment(Segment):
         self.code_block_size = 4 * 2 ** ycb, 4 * 2 ** xcb
 
         if precinct_size is None:
-            self.precinct_size = ((2 ** 15, 2 ** 15))
+            self.precinct_size = np.array(((2 ** 15, 2 ** 15)))
         else:
-            self.precinct_size = precinct_size
+            self.precinct_size = np.array(precinct_size)
 
     def __str__(self):
         msg = Segment.__str__(self)
@@ -1123,7 +1124,7 @@ class CODsegment(Segment):
             cbh=int(self.code_block_size[0]),
             cbw=int(self.code_block_size[1]),
             xform=xform,
-            precinct_size=self.precinct_size,
+            precinct_size=self.precinct_size.tolist(),
             code_block_context=_context_string(self.cstyle)
         )
 
@@ -1667,7 +1668,12 @@ class SIZsegment(Segment):
         try:
             profile = _CAPABILITIES_DISPLAY[self.rsiz]
         except KeyError:
-            profile = f'{self.rsiz} (invalid)'
+            if np.bitwise_and(self.rsiz, 16384):
+                # HTJ2K profile that is uninterpreted
+                profile = f'{self.rsiz}'
+            else:
+                # profile unknown
+                profile = f'{self.rsiz} (invalid)'
         msg = msg.format(
             profile=profile,
             height=self.ysiz, width=self.xsiz,
@@ -1898,7 +1904,7 @@ def _parse_precinct_size(spcod):
         ep2 = (item & 0xF0) >> 4
         ep1 = item & 0x0F
         precinct_size.append((2 ** ep1, 2 ** ep2))
-    return tuple(precinct_size)
+    return np.array(precinct_size)
 
 
 def _context_string(context):
