@@ -4,10 +4,8 @@ Tests for general glymur functionality.
 # Standard library imports ...
 import collections
 import datetime
-import doctest
 import importlib.resources as ir
 from io import BytesIO
-import os
 import pathlib
 import struct
 import unittest
@@ -30,23 +28,6 @@ from .fixtures import OPENJPEG_NOT_AVAILABLE, OPENJPEG_NOT_AVAILABLE_MSG
 from . import fixtures
 
 
-def docTearDown(doctest_obj):  # pragma: no cover
-    glymur.set_option('parse.full_codestream', False)
-
-
-# Doc tests should be run as well.
-def load_tests(loader, tests, ignore):  # pragma: no cover
-    """Should run doc tests as well"""
-    if os.name == "nt":
-        # Can't do it on windows, temporary file issue.
-        return tests
-    if glymur.lib.openjp2.OPENJP2 is not None:
-        tests.addTests(
-            doctest.DocTestSuite('glymur.jp2k', tearDown=docTearDown)
-        )
-    return tests
-
-
 @unittest.skipIf(OPENJPEG_NOT_AVAILABLE, OPENJPEG_NOT_AVAILABLE_MSG)
 @unittest.skipIf(glymur.version.openjpeg_version < '2.3.0',
                  "Requires as least v2.3.0")
@@ -57,6 +38,43 @@ class TestJp2k(fixtures.TestCommon):
         super().setUp()
         glymur.reset_option('all')
 
+    @unittest.skipIf(
+        glymur.version.openjpeg_version < '2.5.0', "Requires as least v2.5.0"
+    )
+    def test_read_htj2k(self):
+        """
+        Scenario:  read an HTJ2K (JPH) file using Jp2k
+
+        Expected response:  The size of the image read is verified.
+        """
+        path = ir.files('tests.data').joinpath('oj-ht-byte.jph')
+        j = Jp2k(path)
+        d = j[:]
+        self.assertEqual(d.shape, (20, 20))
+
+    def test_repr(self):
+        """
+        Scenario:  run repr on Jp2k object
+
+        Expected response:  the representation is verified
+        """
+        j = Jp2k(self.j2kfile)
+        self.assertRegex(repr(j), 'glymur.Jp2k(.*?)')
+
+    def test_write_using_slicing(self):
+        """
+        Scenario: write to a file using slicing then read it back
+
+        Expected results:  no errors, data matches
+        """
+        expected = fixtures.skimage.data.astronaut()
+
+        j1 = Jp2k(self.temp_jp2_filename)
+        j1[:] = expected
+        actual = j1[:]
+
+        np.testing.assert_array_equal(actual, expected)
+
     def test_last_decomposition(self):
         """
         Scenario:  The last decomposition image is requested using [::-1]
@@ -258,31 +276,6 @@ class TestJp2k(fixtures.TestCommon):
 
         np.testing.assert_array_equal(rgb, bgr[:, :, [2, 1, 0]])
 
-    def test_bad_tile_part_pointer(self):
-        """
-        SCENARIO:  A bad SOT marker segment is encountered (Psot value pointing
-        far beyond the end of the EOC marker) when requesting a fully parsed
-        codestream.
-
-        EXPECTED RESULT:  struct.error
-        """
-        with open(self.temp_jp2_filename, 'wb') as ofile:
-            with open(self.jp2file, 'rb') as ifile:
-                # Copy up until Psot field.
-                ofile.write(ifile.read(3350))
-
-                # Write a bad Psot value.
-                ofile.write(struct.pack('>I', 2000000))
-
-                # copy the rest of the file as-is.
-                ifile.seek(3354)
-                ofile.write(ifile.read())
-                ofile.flush()
-
-        j = Jp2k(self.temp_jp2_filename)
-        with self.assertRaises(struct.error):
-            j.get_codestream(header_only=False)
-
     def test_read_differing_subsamples(self):
         """
         SCENARIO:  Attempt to read a file where the components have differing
@@ -295,7 +288,7 @@ class TestJp2k(fixtures.TestCommon):
         with open(self.temp_jp2_filename, mode='wb') as ofile:
             with open(self.jp2file, 'rb') as ifile:
                 # Copy up until codestream box.
-                ofile.write(ifile.read(3223))
+                ofile.write(ifile.read(77))
 
                 # Write the jp2c header and SOC marker.
                 ofile.write(ifile.read(10))
@@ -389,40 +382,6 @@ class TestJp2k(fixtures.TestCommon):
         with self.assertRaises(InvalidJp2kError):
             Jp2k(path)
 
-    def test_file_does_not_exist(self):
-        """
-        Scenario:  The Jp2k construtor is passed a file that does not exist
-        and the intent is reading.
-
-        Expected Result:  FileNotFoundError
-        """
-        # Verify that we error out appropriately if not given an existing file
-        # at all.
-        filename = 'this file does not actually exist on the file system.'
-        j = Jp2k(filename)
-        with self.assertRaises(FileNotFoundError):
-            j[:]
-
-    @unittest.skipIf(
-        not fixtures.HAVE_SCIKIT_IMAGE, fixtures.HAVE_SCIKIT_IMAGE_MSG
-    )
-    def test_write_to_a_file_using_context_manager(self):
-        """
-        SCENARIO:  Write to a file using a context manager, read the data back
-        using a context manager.
-
-        EXPECTED RESULT: the data matches
-        """
-        expected = fixtures.skimage.data.astronaut()
-
-        j1 = Jp2k(self.temp_jp2_filename)
-        j1[:] = expected
-
-        j2 = Jp2k(self.temp_jp2_filename)
-        actual = j2[:]
-
-        np.testing.assert_array_equal(actual, expected)
-
     def test_codestream(self):
         """
         Verify the markers and segments of a JP2 file codestream.
@@ -451,17 +410,19 @@ class TestJp2k(fixtures.TestCommon):
         self.assertEqual(c.segment[1].yrsiz, (1, 1, 1))
 
         self.assertEqual(c.segment[2].marker_id, 'COD')
-        self.assertEqual(c.segment[2].offset, 3282)
+        self.assertEqual(c.segment[2].offset, 136)
         self.assertEqual(c.segment[2].length, 12)
         self.assertEqual(c.segment[2].scod, 0)
         self.assertEqual(c.segment[2].layers, 2)
         self.assertEqual(c.segment[2].code_block_size, (64.0, 64.0))
         self.assertEqual(c.segment[2].prog_order, 0)
         self.assertEqual(c.segment[2].xform, 1)
-        self.assertEqual(c.segment[2].precinct_size, ((32768, 32768)))
+        np.testing.assert_array_equal(
+            c.segment[2].precinct_size, np.array(((32768, 32768)))
+        )
 
         self.assertEqual(c.segment[3].marker_id, 'QCD')
-        self.assertEqual(c.segment[3].offset, 3296)
+        self.assertEqual(c.segment[3].offset, 150)
         self.assertEqual(c.segment[3].length, 7)
         self.assertEqual(c.segment[3].sqcd, 64)
         self.assertEqual(c.segment[3].mantissa, [0, 0, 0, 0])
@@ -474,7 +435,7 @@ class TestJp2k(fixtures.TestCommon):
                          b'Created by OpenJPEG version 2.0.0')
 
         self.assertEqual(c.segment[5].marker_id, 'SOT')
-        self.assertEqual(c.segment[5].offset, 3344)
+        self.assertEqual(c.segment[5].offset, 198)
         self.assertEqual(c.segment[5].length, 10)
         self.assertEqual(c.segment[5].isot, 0)
         self.assertEqual(c.segment[5].psot, 1132173)
@@ -482,18 +443,19 @@ class TestJp2k(fixtures.TestCommon):
         self.assertEqual(c.segment[5].tnsot, 1)
 
         self.assertEqual(c.segment[6].marker_id, 'COC')
-        self.assertEqual(c.segment[6].offset, 3356)
+        self.assertEqual(c.segment[6].offset, 210)
         self.assertEqual(c.segment[6].length, 9)
         self.assertEqual(c.segment[6].ccoc, 1)
         np.testing.assert_array_equal(c.segment[6].scoc,
                                       np.array([0]))
         np.testing.assert_array_equal(c.segment[6].spcoc,
                                       np.array([1, 4, 4, 0, 1]))
-        self.assertEqual(c.segment[6].precinct_size,
-                         ((32768, 32768)))
+        np.testing.assert_array_equal(
+            c.segment[6].precinct_size, np.array(((32768, 32768)))
+        )
 
         self.assertEqual(c.segment[7].marker_id, 'QCC')
-        self.assertEqual(c.segment[7].offset, 3367)
+        self.assertEqual(c.segment[7].offset, 221)
         self.assertEqual(c.segment[7].length, 8)
         self.assertEqual(c.segment[7].cqcc, 1)
         self.assertEqual(c.segment[7].sqcc, 64)
@@ -502,18 +464,20 @@ class TestJp2k(fixtures.TestCommon):
         self.assertEqual(c.segment[7].guard_bits, 2)
 
         self.assertEqual(c.segment[8].marker_id, 'COC')
-        self.assertEqual(c.segment[8].offset, 3377)
+        self.assertEqual(c.segment[8].offset, 231)
         self.assertEqual(c.segment[8].length, 9)
         self.assertEqual(c.segment[8].ccoc, 2)
         np.testing.assert_array_equal(c.segment[8].scoc,
                                       np.array([0]))
         np.testing.assert_array_equal(c.segment[8].spcoc,
                                       np.array([1, 4, 4, 0, 1]))
-        self.assertEqual(c.segment[8].precinct_size,
-                         ((32768, 32768)))
+        np.testing.assert_array_equal(
+            c.segment[8].precinct_size,
+            np.array(((32768, 32768)))
+        )
 
         self.assertEqual(c.segment[9].marker_id, 'QCC')
-        self.assertEqual(c.segment[9].offset, 3388)
+        self.assertEqual(c.segment[9].offset, 242)
         self.assertEqual(c.segment[9].length, 8)
         self.assertEqual(c.segment[9].cqcc, 2)
         self.assertEqual(c.segment[9].sqcc, 64)
@@ -530,7 +494,7 @@ class TestJp2k(fixtures.TestCommon):
         jp2k = Jp2k(self.jp2file)
 
         # top-level boxes
-        self.assertEqual(len(jp2k.box), 5)
+        self.assertEqual(len(jp2k.box), 4)
 
         self.assertEqual(jp2k.box[0].box_id, 'jP  ')
         self.assertEqual(jp2k.box[0].offset, 0)
@@ -547,13 +511,9 @@ class TestJp2k(fixtures.TestCommon):
         self.assertEqual(jp2k.box[2].length, 45)
         self.assertEqual(jp2k.box[2].longname, 'JP2 Header')
 
-        self.assertEqual(jp2k.box[3].box_id, 'uuid')
+        self.assertEqual(jp2k.box[3].box_id, 'jp2c')
         self.assertEqual(jp2k.box[3].offset, 77)
-        self.assertEqual(jp2k.box[3].length, 3146)
-
-        self.assertEqual(jp2k.box[4].box_id, 'jp2c')
-        self.assertEqual(jp2k.box[4].offset, 3223)
-        self.assertEqual(jp2k.box[4].length, 1132296)
+        self.assertEqual(jp2k.box[3].length, 1132296)
 
         # jp2h super box
         self.assertEqual(len(jp2k.box[2].box), 2)
@@ -599,7 +559,7 @@ class TestJp2k(fixtures.TestCommon):
         with open(self.temp_jp2_filename, mode='wb') as tfile:
             with open(self.jp2file, 'rb') as ifile:
                 # Everything up until the jp2c box.
-                write_buffer = ifile.read(3223)
+                write_buffer = ifile.read(77)
                 tfile.write(write_buffer)
 
                 # The L field must be 1 in order to signal the presence of the
@@ -607,7 +567,7 @@ class TestJp2k(fixtures.TestCommon):
                 # (8 bytes for the XL field).
                 length = 1
                 typ = b'jp2c'
-                xlen = 1133427 + 8
+                xlen = 1132296 + 8
                 write_buffer = struct.pack('>I4sQ', int(length), typ, xlen)
                 tfile.write(write_buffer)
 
@@ -620,9 +580,9 @@ class TestJp2k(fixtures.TestCommon):
 
             jp2k = Jp2k(tfile.name)
 
-            self.assertEqual(jp2k.box[4].box_id, 'jp2c')
-            self.assertEqual(jp2k.box[4].offset, 3223)
-            self.assertEqual(jp2k.box[4].length, 1133427 + 8)
+            self.assertEqual(jp2k.box[3].box_id, 'jp2c')
+            self.assertEqual(jp2k.box[3].offset, 77)
+            self.assertEqual(jp2k.box[3].length, 1132296 + 8)
 
     def test_length_field_is_zero(self):
         """
@@ -788,17 +748,6 @@ class TestJp2k(fixtures.TestCommon):
             self.assertEqual(ET.tostring(jp2k.box[3].xml.getroot()),
                              b'<test>this is a test</test>')
 
-    def test_xmp_attribute(self):
-        """Verify the XMP packet in the shipping example file can be read."""
-        j = Jp2k(self.jp2file)
-
-        xmp = j.box[3].data
-        ns0 = '{http://www.w3.org/1999/02/22-rdf-syntax-ns#}'
-        ns1 = '{http://ns.adobe.com/xap/1.0/}'
-        name = '{0}RDF/{0}Description/{1}CreatorTool'.format(ns0, ns1)
-        elt = xmp.find(name)
-        self.assertEqual(elt.text, 'Google')
-
     @unittest.skipIf(OPENJPEG_NOT_AVAILABLE, OPENJPEG_NOT_AVAILABLE_MSG)
     def test_jpx_mult_codestreams_jp2_brand(self):
         """Read JPX codestream when jp2-compatible."""
@@ -819,14 +768,14 @@ class TestJp2k(fixtures.TestCommon):
         with open(self.temp_jp2_filename, mode='wb') as ofile:
             with open(self.jp2file, 'rb') as ifile:
                 # Copy up until codestream box.
-                ofile.write(ifile.read(3223))
+                ofile.write(ifile.read(77))
 
                 # Write the new codestream length (+4) and the box ID.
                 buffer = struct.pack('>I4s', 1132296 + 4, b'jp2c')
                 ofile.write(buffer)
 
                 # Copy up until the EOC marker.
-                ifile.seek(3231)
+                ifile.seek(85)
                 ofile.write(ifile.read(1132286))
 
                 # Write the zero-length reserved segment.
@@ -1099,7 +1048,7 @@ class TestParsing(unittest.TestCase):
         """verify that the main header isn't loaded during normal parsing"""
         # The hidden _main_header attribute should show up after accessing it.
         jp2 = Jp2k(self.jp2file)
-        jp2c = jp2.box[4]
+        jp2c = jp2.box[3]
         self.assertIsNone(jp2c._codestream)
         jp2c.codestream
         self.assertIsNotNone(jp2c._codestream)
