@@ -78,12 +78,15 @@ class TestSuite(fixtures.TestCommon):
             with self.assertRaises(RuntimeError):
                 glymur.set_option('lib.num_threads', 2)
 
-    def test_threads_write_support(self):
+    @unittest.skipIf(
+        glymur.version.openjpeg_version < '2.4.0', "Requires as least v2.3.0"
+    )
+    def test_threads_write_support__ge_2p4(self):
         """
         SCENARIO:  Attempt to encode with threading support.  This feature is
         new as of openjpeg library version 2.4.0.
 
-        EXPECTED RESULT:  In library versions prior to 2.4.0, a warning is
+        EXPECTED RESULT:  No errors.
         issued.
         """
         glymur.set_option('lib.num_threads', 2)
@@ -98,6 +101,24 @@ class TestSuite(fixtures.TestCommon):
             else:
                 self.assertEqual(len(w), 1)
 
+    def test_threads_write_support__eq_2p3(self):
+        """
+        SCENARIO:  Attempt to encode with threading support.  This feature is
+        new as of openjpeg library version 2.4.0.
+
+        EXPECTED RESULT:  In library versions prior to 2.4.0, a warning is
+        issued.
+        """
+        with patch('glymur.jp2k.version.openjpeg_version', new='2.3.0'):
+            glymur.set_option('lib.num_threads', 2)
+
+            with warnings.catch_warnings(record=True) as w:
+                Jp2k(
+                    self.temp_jp2_filename,
+                    data=np.zeros((128, 128), dtype=np.uint8)
+                )
+                self.assertEqual(len(w), 1)
+
     def test_tiff2jp2_num_threads(self):
         """
         Scenario:  The --num-threads option is given on the command line.
@@ -118,3 +139,34 @@ class TestSuite(fixtures.TestCommon):
         Jp2k(self.temp_jp2_filename)
 
         self.assertTrue(True)
+
+    def test_openjpeg_library_too_old_for_threaded_tile_writing(self):
+        """
+        SCENARIO:  Try to create a jp2 file via writing tiles, but while the
+        openjpeg library is not too old for writing, it's too old for threaded
+        writing.  In other words, it's version 2.3.0
+
+        EXPECTED RESULT:  There is a warning, but the image is created.
+        """
+        expected = fixtures.skimage.data.astronaut()
+
+        shape = (
+            expected.shape[0] * 2, expected.shape[1] * 2, expected.shape[2]
+        )
+        tilesize = (expected.shape[0], expected.shape[1])
+
+        glymur.set_option('lib.num_threads', 2)
+        j = Jp2k(
+            self.temp_j2k_filename, shape=shape, tilesize=tilesize,
+        )
+
+        with patch('glymur.version.openjpeg_version', new='2.3.0'):
+            with self.assertWarns(UserWarning):
+                for tw in j.get_tilewriters():
+                    tw[:] = expected
+
+        expected = np.concatenate((expected, expected), axis=0)
+        expected = np.concatenate((expected, expected), axis=1)
+        actual = j[:]
+
+        np.testing.assert_array_equal(actual, expected)
